/** 
 * Copyright (c) 2007-2008, Regents of the University of Colorado 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * Neither the name of the University of Colorado at Boulder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. 
 */
package org.cleartk.classifier.liblinear;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;

import org.cleartk.classifier.liblinear.model.LIBLINEARModel;
import org.cleartk.classifier.libsvm.GenericLIBSVMClassifierBuilder;

import de.bwaldvogel.liblinear.Train;

/**
 * <br>
 * Copyright (c) 2007-2008, Regents of the University of Colorado <br>
 * All rights reserved.
 */

public class LIBLINEARBooleanOutcomeClassifierBuilder extends
    GenericLIBSVMClassifierBuilder<LIBLINEARBooleanOutcomeClassifier, Boolean, Boolean, LIBLINEARModel> {

  @Override
  public String getTrainingDataName() {
    return "training-data.liblinear";
  }

  @Override
  public String getModelName() {
    return "model.liblinear";
  }

  @Override
  public void trainClassifier(File dir, String... args) throws Exception {
    args = Arrays.copyOf(args, args.length + 2);
    args[args.length - 2] = this.getTrainingDataFile(dir).getPath();
    args[args.length - 1] = this.getModelFile(dir).getPath();
    Train.main(args);
  }

  @Override
  protected LIBLINEARBooleanOutcomeClassifier newClassifier() {
    return new LIBLINEARBooleanOutcomeClassifier(this.featuresEncoder, this.outcomeEncoder, this.model);
  }

  @Override
  protected LIBLINEARModel loadModel(InputStream inputStream) throws IOException {
    return LIBLINEARModel.fromInputStream(inputStream);
  }

}
